# File: TreeVocabulary.py
"""\
A Tree Vocabulary is a container for hierachical key/value pairs.

RCS-ID $Id: TreeVocabulary.py 3172 2004-10-13 21:18:00Z yenzenz $
"""
# Copyright (c) 2004-2006 by BlueDynamics Alliance - Klein & Partner KEG, Austria
#
# BSD-like licence, see LICENCE.txt
#
__author__  = 'Jens Klein <jens@bluedynamics.com>'
__docformat__ = 'plaintext'

from Products.ATVocabularyManager.config import *
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.atapi import *

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import AddPortalContent
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.interfaces.vocabulary import IVocabulary
from Products.Archetypes.utils import OrderedDict
try:
    from Products.Archetypes.lib.vocabulary import DisplayList
except ImportError:
    from Products.Archetypes.utils import DisplayList

from Products.ATVocabularyManager.tools import registerVocabularyContainer
from Products.ATVocabularyManager.types.simple import SimpleVocabulary

schema = SimpleVocabulary.schema + Schema((
   BooleanField('ShowLeavesOnly'),
)
)

class TreeVocabulary(SimpleVocabulary):
    security = ClassSecurityInfo()

    schema = schema

    portal_type = meta_type = 'TreeVocabulary'
    archetype_name = 'Tree Vocabulary'   #this name appears in the 'add' box

    def getDisplayList(self, instance):
        """ returns an object of class DisplayList as defined in
            Products.Archetypes.utils

            The instance of the content class is given as parameter.
        """
        dl = DisplayList()
        self._appendToDisplayList(dl, self.getVocabularyDict(instance), None)
        return dl

    def _appendToDisplayList(self, displaylist, vdict, valueparent):
        """ append subtree to flat display list
        """
        print vdict
        for key in vdict.keys():
            if type(vdict[key]) == type((1,2)):
                value  = vdict[key][0]
                subdict= vdict[key][1] or None
            else:
                value  = vdict[key]
                subdict= None
            if valueparent:
                value = '%s - %s' % (valueparent, value)
            if not self.showLeafsOnly() or subdict:
                displaylist.add(key,value)
            if subdict:
                self._appendToDisplayList(displaylist,subdict,value)





    def getVocabularyDict(self, instance=None):
        """returns a vocabulary dictionary as defined in the interface
        """
        
        # see if we simply can return all objects and their
        # subtree, or if we have to use the correct translations

        if self.isLinguaPloneInstalled():
            # find out the currently used Language
            if not instance is None:
                lang = instance.getLanguage()
            else:  
                # we try to retrieve the current language
                langtool = getToolByName(self,'portal_languages')
                lang = langtool.getPreferredLanguage()
                        
            return self._getTranslatedVocabularyDict(lang)
        else:
            # we don't need to care about languages, and can simply
            # return all contentObjects
            return self._getUntranslatedVocabularyDict(instance)



    def _getTranslatedVocabularyDict(self, lang):
        """returns a vocabulary dict using the titles of the
        translations for language ``lang``
        
        Only canonical objects are used to build up this
        dictionary. If available, the translation's titles
        are used.
        """
        vdict = OrderedDict()
        
        for obj in self.contentValues():
            if obj.isCanonical():
                # we only add the canonical objects to this dict
                key = obj.getTermKey()
                # but use the title of the appropriate translation
                # if it is available (getTermValue is LP aware)
                vsubdict=obj._getTranslatedVocabularyDict(lang)
                vdict[key] = (obj.getTermValue(lang=lang), vsubdict)
        return vdict
                

    def _getUntranslatedVocabularyDict(self, instance):
        """returns a vocabulary dictionary as defined in the interface
        """

        vdict = OrderedDict()
        for obj in self.contentValues():
            vsubdict=obj.getVocabularyDict(instance)
            vdict[obj.getVocabularyKey()] = ( \
                obj.getVocabularyValue(instance),
                vsubdict
            )
        return vdict
       
    # check for linguaplone
    def _isLinguaPloneInstalled(self):
        """ checks if LinguaPlone is installed """
        return self.portal_quickinstaller.isProductInstalled('LinguaPlone')
        
    def showLeafsOnly(self):
        """ indicates if only leaves should be shown """
        if hasattr(self,'getShowLeafsOnly'):
            return self.getShowLeafsOnly()
        return None

    def isFlat(self):
        """ indicates if tree or flat """
        return 0



registerType(TreeVocabulary)
registerVocabularyContainer(TreeVocabulary)
# end of class TreeVocabulary
